#include "at32f403a_407.h"
#include "input.h"
#include "FreeRTOS.h"
#include "task.h"
#include <stdio.h>

in_t inputs[INPUT_NUMBER] = {
    {GPIOB, GPIO_PINS_11, 1, GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE11, 0},
    {GPIOB, GPIO_PINS_10, 1, GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE10, 0},
    {GPIOB, GPIO_PINS_1,  1, GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE1,  0},
    {GPIOB, GPIO_PINS_0,  1, GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE0,  0},
    {GPIOA, GPIO_PINS_3,  1, GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE3,  0},
    {GPIOA, GPIO_PINS_2,  1, GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE2,  0},
    {GPIOA, GPIO_PINS_1,  1, GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE1,  0},
    {GPIOA, GPIO_PINS_0,  1, GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE0,  0}
};

//
void in_init(in_t *input)
{
    gpio_init_type gpio_init_struct;
    exint_init_type exint_init_struct;

    // Вход
    if (input->mode == 0)
    {
        gpio_default_para_init(&gpio_init_struct);
        
        gpio_init_struct.gpio_pull           = GPIO_PULL_NONE;  
        gpio_init_struct.gpio_mode           = GPIO_MODE_INPUT;  
        gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
        gpio_init_struct.gpio_pins           = input->pin;
        gpio_init(input->port, &gpio_init_struct); 
    }
    // Счетный вход
    else if (input->mode == 1)
    {
        
    }

}

//
void in_exint_init(void)
{
    exint_init_type exint_init_struct;
    
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE11); // 11
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE10); // 10
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE1);  // 1
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOB, GPIO_PINS_SOURCE0);  // 0
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE3);  // 3
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE2);  // 2
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE1);  // 1
    gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOA, GPIO_PINS_SOURCE0);  // 0
    
    exint_default_para_init(&exint_init_struct);
    
    exint_init_struct.line_enable = TRUE;
    exint_init_struct.line_mode = EXINT_LINE_INTERRUPUT;
    //exint_init_struct.line_select = EXINT_LINE_0 | EXINT_LINE_1 | EXINT_LINE_2 |
    //                                EXINT_LINE_3 | EXINT_LINE_10 | EXINT_LINE_11; 
    
    //exint_init_struct.line_select = EXINT_LINE_11 | EXINT_LINE_10 | EXINT_LINE_1; 
    exint_init_struct.line_select = EXINT_LINE_1; 
    
    exint_init_struct.line_polarity = EXINT_TRIGGER_RISING_EDGE;
    exint_init(&exint_init_struct);
    
    nvic_irq_enable(EXINT0_IRQn, 5, 0);
    nvic_irq_enable(EXINT1_IRQn, 5, 0);
    nvic_irq_enable(EXINT2_IRQn, 5, 0);
    nvic_irq_enable(EXINT3_IRQn, 5, 0);
    nvic_irq_enable(EXINT4_IRQn, 5, 0);
    nvic_irq_enable(EXINT15_10_IRQn, 5, 0);
}

//
void io_test(void)
{
    printf("IN_1: %u IN_2: %u IN_3: %u IN_4: %u IN_5: %u IN_6: %u IN_7: %u IN_8: %u\r\n", 
           gpio_input_data_bit_read(inputs[0].port, inputs[0].pin),
           gpio_input_data_bit_read(inputs[1].port, inputs[1].pin),
           gpio_input_data_bit_read(inputs[2].port, inputs[2].pin),
           gpio_input_data_bit_read(inputs[3].port, inputs[3].pin),
           gpio_input_data_bit_read(inputs[4].port, inputs[4].pin),
           gpio_input_data_bit_read(inputs[5].port, inputs[5].pin),
           gpio_input_data_bit_read(inputs[6].port, inputs[6].pin), 
           gpio_input_data_bit_read(inputs[7].port, inputs[7].pin));
}

//
void input_task(void *params)
{
    for (;;)
    {
        for (int i = 0; i < INPUT_NUMBER; i++)
        {
            // Режим обычного входа
            if (inputs[i].mode == 0)
            {
                input_state[i] = (uint16_t)gpio_input_data_bit_read(inputs[i].port, inputs[i].pin);
            }
        }
            
        vTaskDelay(100);
    }
}


void EXINT0_IRQHandler(void)
{
    if(exint_flag_get(EXINT_LINE_0) != RESET)
    {
        exint_flag_clear(EXINT_LINE_0);
    }
}

void EXINT1_IRQHandler(void)
{
    if(exint_flag_get(EXINT_LINE_1) != RESET)   // IN_3 GPIOB_1
    {
        exint_flag_clear(EXINT_LINE_1);
    }
}

void EXINT2_IRQHandler(void)
{
    if(exint_flag_get(EXINT_LINE_0) != RESET)
    {
        exint_flag_clear(EXINT_LINE_0);
    }
}

void EXINT3_IRQHandler(void)
{
    if(exint_flag_get(EXINT_LINE_0) != RESET)
    {
        exint_flag_clear(EXINT_LINE_0);
    }
}

void EXINT4_IRQHandler(void)
{
    if(exint_flag_get(EXINT_LINE_0) != RESET)
    {
        exint_flag_clear(EXINT_LINE_0);
    }
}

void EXINT15_10_IRQHandler(void)
{
    if (exint_flag_get(EXINT_LINE_10) != RESET)  // IN_2 GPIOB_10
    {
        exint_flag_clear(EXINT_LINE_10);
    }
    if (exint_flag_get(EXINT_LINE_11) != RESET)  // IN_1 GPIOB_11
    {
        exint_flag_clear(EXINT_LINE_11);
    }
    
}
